# dbph - convert complex (real,imag) to (dB,phase)

    subroutine dbph(y,npts)
    real y(*)
    data yrf/0./

    if (npts < 1) {
	yrf = 0
	}
    else {
	call cvln(y,npts)                     # complex log
	call cvuwp(yrf,y,npts)                # unwrap phase
	sc=20./alog(10.)
	call vsml(sc,y(1),2,npts)
	# scale magnitude
	yrf=y(2)                              # save phase ref.
	}

	return
	end

# dbgd - convert complex (real,imag) to (dB,groupdelay)

    subroutine dbgd(y,npts,df)
    real y(*)
    data pi/3.1415927/

    call dbph(y,npts)
    if(df<=0.)
	return
    sc=.5/(2.*pi*df)
    n2=npts*2
    phb=y(4)
    for(i=2;i<n2;i=i+2)
	{
	phc=y(i)
	phf=y(i+2)
	y(i)=(phb-phf)*sc
	phb=phc
	}
    y(i)=0.
    return
    end

# cvln - complex vector natural log

    subroutine cvln(y,npts)
    real y(*)
    real ymx,ymnsq
    data ymx/1.e19/

    ymnsq=1/(ymx*ymx)
    ylnmx=alog(ymx)
    for(i=1; i<=npts; i=i+1) {
        ir = 2 * i - 1
        ii = 2 * i
	yr=y(ir)
	yi=y(ii)
	if(abs(yr)>ymx | abs(yi)>ymx) {
	    yr=yr/ymx
	    yi=yi/ymx
	    ymsq=yr*yr+yi*yi
	    yln=.5*alog(ymsq)+ylnmx
	    yph=atan2(yi,yr)
	    }
	else {
	    ymsq=yr*yr+yi*yi
	    if(ymsq>ymnsq) {
		yln=.5*alog(ymsq)
		yph=atan2(yi,yr)
		}
	    else {
		yln=-2*ylnmx
		yph=0.
		}
	    }
	y(ir)=yln
	y(ii)=yph
	}
    return
    end

# vsml - vector scaler multiply

    subroutine vsml(sc,y,incr,npts)
    real y(*)

    do i=1,npts {
	j=1+(i-1)*incr
	y(j)=sc*y(j)
	}
    return
    end

# cvuwp - complex vector unwrap phase

    subroutine cvuwp(yrf,y,npts)
    real y(*)

    # definitions
    tpi=2.*3.1415927
    npts2=npts*2
    phref=yrf

    for(j=2; j<=npts2; j=j+2) {
	ph=y(j)
	y(j) = ph + nint( (phref-ph)/tpi )*tpi
	phref=y(j)
	}
    return
    end

