# fdmgo - frequency domain finite difference model
    include fdm.h
    define(DATAOUT,1)

    subroutine fdmgo
    integer init,opnfil
    include cparms
    include cio
    data init/0/

    if(init==0) {
	if(opnfil(DATAOUT,'fdm.dat','unformatted')==EOF)
	    return
	call fdate(date)
	call wrhdr
	init=1
	}
    f=freq(1)
    write(ERROUT,200)m,n,xl,zh,f
    call fdmodl(f)
    call wrsoln(f)
    call inimp
    call powrgn(f)
    return
    200   format(' *** fdmodl:',2i6,3f10.2,i5)
    end

# opnfil

	integer function opnfil(un,fi,fo)
	character*(*) fi,fo
	integer un

	open(unit=un,file=fi,status='unknown',form=fo,err=1)
	return(un)
	1 continue
	write(ERROUT,*)'can''t open: ',fi
	return(EOF)
	end

# bells - send signal to console

   subroutine bells(num)
   #for(i=1; i<=num; i=i+1)
      #call fputc(ERROUT,char(7))
   return
   200 format(' ',1a1)
   end

# fq - compute frequency and q of poles and zeros

     subroutine fq(k,r,m)
     real k,r,m,tpi,eps,f,q
     data tpi,eps/6.2832,1.e-9/

     f=sqrt( k/amax1(m,eps) )/tpi
     q=sqrt( k*m )/r
     write(ERROUT,200)k,r,m,f,q
     return
     200 format(' k,r,m,f,q =',1p3g10.3,0pf10.0,f8.2)
     end

# inimp - input impedance

    subroutine inimp
    complex vs,zpc
    include cparms
    include cucon
    include carrays

    vs = yf(1)*(p(1)-p(2))		# volume velocity at stapes
    zpc = p(1)/vs			# cochlear input impedance
    zpcm=cabs(zpc)
    zpcp=atan2(aimag(zpc),real(zpc))*(180./3.1415927)
    write(ERROUT,*)'zpc =',zpcm,zpcp
    #zwkb=sqrt(2*rho*zh*k1o*(gpo/bwo))/aco
    write(ERROUT,*)'zwkb =',zwkb
    return
    end

# wrhdr - write header to data file

    subroutine wrhdr
    include cparms

    write(DATAOUT)title
    write(DATAOUT)date
    write(DATAOUT)laba
    write(DATAOUT)labb
    write(DATAOUT)labc
    for(i=1; i<=3; i=i+1)
	write(DATAOUT)dbfs(i),dbmx(i),phfs(i),phmx(i)
    end

# wrsoln - write solution to data file

    subroutine wrsoln(fr)
    include cio
    include carrays
    include cucon
    include cparms

    write(DATAOUT)m,n,xl,yw,zh,fr
    # pressure
    write(DATAOUT)(p(i),i=1,n)
    a(1)=0
    a(n)=0
    call admit(fr,14)		# BM displacement
    for(i=2;i<n;i=i+1)
	a(i)=y(i)*p(i)
    write(DATAOUT)(a(i),i=1,n)
    call admit(fr,3)		# IHC displacement
    for(i=2;i<n;i=i+1)
	a(i)=y(i)*p(i)
    write(DATAOUT)(a(i),i=1,n)
    imax=0.
    amax=0.
    for(i=2;i<n;i=i+1) {
	amag=cabs(a(i))
	if(amax<amag) {
	    imax=i-1
	    amax=amag
	    }
	}
    xmax=imax*dx
    adb = 20*log10(amax)
    write(ERROUT,*)'IHCmax =',amax,' (',adb,' dB) at x =',xmax,' cm ', imax
    return
    end

# powrgn - compute power gain

    subroutine powrgn(fr)
    complex vs
    include cparms
    include cucon
    include carrays
    data w/1e11/	# convert units from dyne*cm/s to attowatt

    vs = yf(1)*(p(1)-p(2))	    # volume velocity at stapes
    pinp = 0.5*real(p(1)*conjg(vs))
    call admit(fr,0)		    # partition admittance
    pout = 0
    pact = 0
    pmax = 0
    ipmx = 0
    rmax = 0
    irmx = 0
    for(i=2;i<n;i=i+1) {
	xol=(i-1)/float(n-1)
	powr = 0.5*real(y(i))*cabs(p(i))**2
	if (powr > 0)
	    pout = pout + powr
	else if (powr < 0) {
	    pact = pact - powr
	    if(pmax < -powr) {
		pmax = -powr
		ipmx = i
		}
	    }
	r = real(1/y(i))
	if (rmax > r) {
	    rmax = r
	    irmx = i
	    }
	}
    write(ERROUT,*)'pinp,pact,pout =',pinp*w,pact*w,pout*w,' (aW)'
    padd = pinp+pact
    if (padd <= 0)
	return
    pgain = 10*log10(padd/pinp)
    ploss = 10*log10(pout/padd)
    write(ERROUT,*)'powrgn: pgain =',pgain,' dB, ploss =',ploss,' dB'
    xx = xp(ipmx)*xl
    write(ERROUT,*)'peak power =',pmax*w/dx,' (aW/cm) at x =',xx,' (cm)'
    xx = xp(irmx)*xl
    write(ERROUT,*)'max. neg. damp. =',rmax,' at x =',xx,' (cm)'
    return
    end
